unit FileView;
{$X+,V-}
interface

uses Drivers, Objects, Views, Dialogs, Dos, MsgBox, Memory, App;

type  PLineCollection = ^TLineCollection;
      TLineCollection = object(TCollection)
        procedure FreeItem(P: Pointer); virtual;
      end;

      PFileViewer = ^TFileViewer;
      TFileViewer = object(TScroller)
        FileName:   PString;
        FileLines:  PCollection;
        IsValid:    Boolean;
        constructor Init(var Bounds: TRect; AHScrollBar, AVScrollBar: PScrollBar; const AFileName: PathStr);
        destructor  Done; virtual;
        procedure   Draw; virtual;
        procedure   ReadFile(const FName: PathStr);
        procedure   SetState(AState: Word; Enable: Boolean); virtual;
        function    Valid(Command: Word): Boolean; virtual;
      end;

      PTextWindow = ^TTextWindow;
      TTextWindow = object(TDialog)
        constructor Init(R: TRect; const FileName: PathStr);
        function    GetPalette:PPalette; virtual;
      end;

procedure ViewFile(const FileName: FNameStr);

implementation

procedure TLineCollection.FreeItem(P: Pointer);
begin
  DisposeStr(P);
end;

constructor TFileViewer.Init(var Bounds: TRect; AHScrollBar,AVScrollBar: PScrollBar; const AFileName: PathStr);
begin
  TScroller.Init(Bounds, AHScrollbar, AVScrollBar);
  GrowMode := gfGrowHiX + gfGrowHiY;
  FileName := nil;
  ReadFile(AFileName);
end;

destructor TFileViewer.Done;
begin
  Dispose(FileLines, Done);
  DisposeStr(FileName);
  TScroller.Done;
end;

procedure TFileViewer.Draw;
var  B: TDrawBuffer;
     C: Byte;
     I: Integer;
     S: String;
     P: PString;
begin
  C := GetColor(1);
  for I := 0 to Size.Y - 1 do begin
    MoveChar(B, ' ', C, Size.X);
    if Delta.Y + I < FileLines^.Count then begin
      P := FileLines^.At(Delta.Y + I);
      if P <> nil then S := Copy(P^, Delta.X + 1, Size.X)
      else S := '';
      MoveStr(B, S, C);
    end;
    WriteLine(0, I, Size.X, 1, B);
  end;
end;

procedure TFileViewer.ReadFile(const FName: PathStr);
var  FileToView: Text;
     Line      : String;
     E         : TEvent;
     MaxLength : Integer;
begin
  IsValid := True;
  if FileName <> nil then DisposeStr(FileName);
  FileName := NewStr(FName);
  FileLines := New(PLineCollection, Init(5,5));
  {$I-}
  Assign(FileToView, FName);
  Reset(FileToView);
  if IOResult <> 0 then begin
    MessageBox(#3'  ந ⨥ 䠩'#13#3+FName, nil, mfError + mfOkButton);
    IsValid := False;
  end
  else begin
    MaxLength:=0;
    While not eof(FileToView) and not LowMemory do begin
      ReadLn(FileToView, Line);
      if Length(Line)>MaxLength then MaxLength:=length(Line);
      FileLines^.Insert(NewStr(Line));
    end;
    Close(FileToView);
  end;
  {$I+}
  Limit.X := MaxLength;
  Limit.Y := FileLines^.Count;
end;

procedure TFileViewer.SetState(AState: Word; Enable: Boolean);
begin
  TScroller.SetState(AState, Enable);
  if Enable and (AState and sfExposed <> 0) then SetLimit(Limit.X, Limit.Y);
end;

function TFileViewer.Valid(Command: Word): Boolean;
begin
  Valid := IsValid;
end;

{ TTextWindow }
constructor TTextWindow.Init(R: TRect; const FileName: PathStr);
begin
  inherited Init(R, Filename);
  GetExtent(R);
  R.Grow(-1, -1);
  Insert(New(PFileViewer, Init(R,
         StandardScrollBar(sbHorizontal + sbHandleKeyboard),
         StandardScrollBar(sbVertical   + sbHandleKeyboard), Filename)));
end;

function TTextWindow.GetPalette:PPalette;
const P: String[Length(CGrayWindow)] = CCyanWindow;
begin
  GetPalette := @P;
end;

procedure ViewFile(const FileName: FNameStr);
var T: PTextWindow;
    R: TRect;
begin
  R.Assign(0,0,80,23);
  T          := New(PTextWindow, Init(R, FileName));
  T^.Options := T^.Options or ofCentered;
  DeskTop^.ExecView(T);
end;

end.

